@@ -1,9 +1,10 @@
-{-# LANGUAGE CPP                        #-}
-{-# LANGUAGE DeriveDataTypeable         #-}
-{-# LANGUAGE DeriveGeneric              #-}
-{-# LANGUAGE GADTs                      #-}
+{-# LANGUAGE CPP #-}
+{-# LANGUAGE DeriveDataTypeable #-}
+{-# LANGUAGE DeriveGeneric #-}
+{-# LANGUAGE GADTs #-}
 {-# LANGUAGE GeneralizedNewtypeDeriving #-}
-{-# LANGUAGE StandaloneDeriving         #-}
+{-# LANGUAGE StandaloneDeriving #-}
+
 #ifdef LIFT_COMPAT
 {-# LANGUAGE TemplateHaskell            #-}
 #else
@@ -12,27 +13,27 @@
 module URI.ByteString.Types where
 
 -------------------------------------------------------------------------------
-import           Data.ByteString (ByteString)
+import Data.ByteString (ByteString)
 import qualified Data.Map.Strict as M
-import           Data.Monoid
-import           Data.Semigroup (Semigroup)
-import           Data.Typeable
-import           Data.Word
-import           GHC.Generics
-import           Instances.TH.Lift()
+import Data.Monoid
+import Data.Semigroup as Semigroup
+import Data.Typeable
+import GHC.Generics
+import Instances.TH.Lift ()
 -------------------------------------------------------------------------------
-import           Prelude
+import Prelude
+
 -------------------------------------------------------------------------------
 #ifdef LIFT_COMPAT
-import           Language.Haskell.TH.Syntax()
 import           Language.Haskell.TH.Lift
+import           Language.Haskell.TH.Syntax ()
 #else
 import           Language.Haskell.TH.Syntax
 #endif
 
 -- | Required first component to referring to a specification for the
 -- remainder of the URI's components, e.g. "http" or "https"
-newtype Scheme = Scheme { schemeBS :: ByteString }
+newtype Scheme = Scheme {schemeBS :: ByteString}
   deriving (Show, Eq, Generic, Typeable, Ord)
 
 #ifdef LIFT_COMPAT
@@ -42,7 +43,7 @@ deriving instance Lift Scheme
 #endif
 
 -------------------------------------------------------------------------------
-newtype Host = Host { hostBS :: ByteString }
+newtype Host = Host {hostBS :: ByteString}
   deriving (Show, Eq, Generic, Typeable, Ord)
 
 #ifdef LIFT_COMPAT
@@ -52,9 +53,10 @@ deriving instance Lift Host
 #endif
 
 -------------------------------------------------------------------------------
+
 -- | While some libraries have chosen to limit this to a Word16, the
 -- spec only specifies that the string be comprised of digits.
-newtype Port = Port { portNumber :: Int }
+newtype Port = Port {portNumber :: Int}
   deriving (Show, Eq, Generic, Typeable, Ord)
 
 #ifdef LIFT_COMPAT
@@ -64,10 +66,11 @@ deriving instance Lift Port
 #endif
 
 -------------------------------------------------------------------------------
-data UserInfo = UserInfo {
-      uiUsername :: ByteString
-    , uiPassword :: ByteString
-    } deriving (Show, Eq, Generic, Typeable, Ord)
+data UserInfo = UserInfo
+  { uiUsername :: ByteString,
+    uiPassword :: ByteString
+  }
+  deriving (Show, Eq, Generic, Typeable, Ord)
 
 #ifdef LIFT_COMPAT
 deriveLift ''UserInfo
@@ -76,11 +79,12 @@ deriving instance Lift UserInfo
 #endif
 
 -------------------------------------------------------------------------------
-data Authority = Authority {
-      authorityUserInfo :: Maybe UserInfo
-    , authorityHost     :: Host
-    , authorityPort     :: Maybe Port
-    } deriving (Show, Eq, Generic, Typeable, Ord)
+data Authority = Authority
+  { authorityUserInfo :: Maybe UserInfo,
+    authorityHost :: Host,
+    authorityPort :: Maybe Port
+  }
+  deriving (Show, Eq, Generic, Typeable, Ord)
 
 #ifdef LIFT_COMPAT
 deriveLift ''Authority
@@ -89,8 +93,8 @@ deriving instance Lift Authority
 #endif
 
 -------------------------------------------------------------------------------
-newtype Query = Query { queryPairs :: [(ByteString, ByteString)] }
-              deriving (Show, Eq, Semigroup, Monoid, Generic, Typeable, Ord)
+newtype Query = Query {queryPairs :: [(ByteString, ByteString)]}
+  deriving (Show, Eq, Semigroup.Semigroup, Monoid, Generic, Typeable, Ord)
 
 #ifdef LIFT_COMPAT
 deriveLift ''Query
@@ -99,7 +103,7 @@ deriving instance Lift Query
 #endif
 
 -------------------------------------------------------------------------------
-data Absolute deriving(Typeable)
+data Absolute deriving (Typeable)
 
 #ifdef LIFT_COMPAT
 deriveLift ''Absolute
@@ -108,7 +112,7 @@ deriving instance Lift Absolute
 #endif
 
 -------------------------------------------------------------------------------
-data Relative deriving(Typeable)
+data Relative deriving (Typeable)
 
 #ifdef LIFT_COMPAT
 deriveLift ''Relative
@@ -117,24 +121,32 @@ deriving instance Lift Relative
 #endif
 
 -------------------------------------------------------------------------------
+
 -- | Note: URI fragment does not include the #
 data URIRef a where
-  URI :: { uriScheme :: Scheme
-         , uriAuthority :: Maybe Authority
-         , uriPath :: ByteString
-         , uriQuery :: Query
-         , uriFragment :: Maybe ByteString
-         } -> URIRef Absolute
-  RelativeRef :: { rrAuthority :: Maybe Authority
-                 , rrPath :: ByteString
-                 , rrQuery :: Query
-                 , rrFragment :: Maybe ByteString
-                 } -> URIRef Relative
+  URI ::
+    { uriScheme :: Scheme,
+      uriAuthority :: Maybe Authority,
+      uriPath :: ByteString,
+      uriQuery :: Query,
+      uriFragment :: Maybe ByteString
+    } ->
+    URIRef Absolute
+  RelativeRef ::
+    { rrAuthority :: Maybe Authority,
+      rrPath :: ByteString,
+      rrQuery :: Query,
+      rrFragment :: Maybe ByteString
+    } ->
+    URIRef Relative
 
 deriving instance Show (URIRef a)
+
 deriving instance Eq (URIRef a)
+
 -- deriving instance Generic (URIRef a)
 deriving instance Ord (URIRef a)
+
 #ifdef LIFT_COMPAT
 deriveLift ''URIRef
 #else
@@ -148,58 +160,62 @@ deriving instance Typeable URIRef
 -------------------------------------------------------------------------------
 type URI = URIRef Absolute
 
-
 -------------------------------------------------------------------------------
 type RelativeRef = URIRef Relative
 
-
 -------------------------------------------------------------------------------
+
 -- | Options for the parser. You will probably want to use either
 -- "strictURIParserOptions" or "laxURIParserOptions"
-data URIParserOptions = URIParserOptions {
-      upoValidQueryChar :: Word8 -> Bool
-    }
-
+data URIParserOptions = URIParserOptions
+  { upoLaxQueryParsing :: Bool
+  }
 
 -------------------------------------------------------------------------------
-data URINormalizationOptions = URINormalizationOptions {
-      unoDowncaseScheme    :: Bool
-    -- ^ hTtP -> http
-    , unoDowncaseHost      :: Bool
-    -- ^ eXaMpLe.org -> example.org
-    , unoDropDefPort       :: Bool
-    -- ^ If the scheme is known and the port is the default (e.g. 80 for http) it is removed.
-    , unoSlashEmptyPath    :: Bool
-    -- ^ If the path is empty, set it to \/
-    , unoDropExtraSlashes  :: Bool
-    -- ^ Rewrite path from \/foo\/\/bar\/\/\/baz to \/foo\/bar\/baz
-    , unoSortParameters    :: Bool
-    -- ^ Sorts parameters by parameter name
-    , unoRemoveDotSegments :: Bool
-    -- ^ Remove dot segments as per <https://tools.ietf.org/html/rfc3986#section-5.2.4 RFC3986 Section 5.2.4>
-    , unoDefaultPorts      :: M.Map Scheme Port
-    -- ^ Map of known schemes to their default ports. Used when 'unoDropDefPort' is enabled.
-    } deriving (Show, Eq)
-
+data URINormalizationOptions = URINormalizationOptions
+  { -- | hTtP -> http
+    unoDowncaseScheme :: Bool,
+    -- | eXaMpLe.org -> example.org
+    unoDowncaseHost :: Bool,
+    -- | If the scheme is known and the port is the default (e.g. 80 for http) it is removed.
+    unoDropDefPort :: Bool,
+    -- | If the path is empty, set it to \/
+    unoSlashEmptyPath :: Bool,
+    -- | Rewrite path from \/foo\/\/bar\/\/\/baz to \/foo\/bar\/baz
+    unoDropExtraSlashes :: Bool,
+    -- | Sorts parameters by parameter name
+    unoSortParameters :: Bool,
+    -- | Remove dot segments as per <https://tools.ietf.org/html/rfc3986#section-5.2.4 RFC3986 Section 5.2.4>
+    unoRemoveDotSegments :: Bool,
+    -- | Map of known schemes to their default ports. Used when 'unoDropDefPort' is enabled.
+    unoDefaultPorts :: M.Map Scheme Port
+  }
+  deriving (Show, Eq)
 
 -------------------------------------------------------------------------------
+
 -- | URI Parser Types
--------------------------------------------------------------------------------
-
-
-data SchemaError = NonAlphaLeading -- ^ Scheme must start with an alphabet character
-                 | InvalidChars    -- ^ Subsequent characters in the schema were invalid
-                 | MissingColon    -- ^ Schemas must be followed by a colon
-                 deriving (Show, Eq, Read, Generic, Typeable)
-
 
 -------------------------------------------------------------------------------
-data URIParseError = MalformedScheme SchemaError
-                   | MalformedUserInfo
-                   | MalformedQuery
-                   | MalformedFragment
-                   | MalformedHost
-                   | MalformedPort
-                   | MalformedPath
-                   | OtherError String -- ^ Catchall for unpredictable errors
-                   deriving (Show, Eq, Generic, Read, Typeable)
+
+data SchemaError
+  = -- | Scheme must start with an alphabet character
+    NonAlphaLeading
+  | -- | Subsequent characters in the schema were invalid
+    InvalidChars
+  | -- | Schemas must be followed by a colon
+    MissingColon
+  deriving (Show, Eq, Read, Generic, Typeable, Enum, Bounded)
+
+-------------------------------------------------------------------------------
+data URIParseError
+  = MalformedScheme SchemaError
+  | MalformedUserInfo
+  | MalformedQuery
+  | MalformedFragment
+  | MalformedHost
+  | MalformedPort
+  | MalformedPath
+  | -- | Catchall for unpredictable errors
+    OtherError String
+  deriving (Show, Eq, Generic, Read, Typeable)
