@@ -1,19 +1,21 @@
 module Main (main) where
 
 -------------------------------------------------------------------------------
-import           Test.Tasty
--------------------------------------------------------------------------------
-import qualified URI.ByteStringTests
-import qualified URI.ByteStringQQTests
+import Test.Tasty
 -------------------------------------------------------------------------------
 
+import qualified URI.ByteStringQQTests
+import qualified URI.ByteStringTests
+
+-------------------------------------------------------------------------------
 
 main :: IO ()
 main = defaultMain testSuite
 
 testSuite :: TestTree
-testSuite = testGroup "uri-bytestring"
-  [
-    URI.ByteStringTests.tests
-  , URI.ByteStringQQTests.tests
-  ]
+testSuite =
+  testGroup
+    "uri-bytestring"
+    [ URI.ByteStringTests.tests,
+      URI.ByteStringQQTests.tests
+    ]
